import os.path
from uuid import uuid4

from django.conf import settings
from django.contrib.auth.models import (
    AbstractBaseUser,
    BaseUserManager,
    PermissionsMixin,
)
from django.core.exceptions import ValidationError
from django.core.mail import send_mail
from django.db import models
from django.utils import timezone
from django.utils.text import slugify

from benevalibre.utils.images import resize_image_to_file

AVATAR_THUMB_SIZE = getattr(settings, 'AVATAR_THUMB_SIZE', (45, 45))


class UserManager(BaseUserManager):
    use_in_migrations = True

    def create_user(self, email, password=None, **extra_fields):
        """
        Crée et enregistre un compte utilisateur avec l'adresse mail et le
        mot de passe donnés.
        """
        extra_fields.setdefault('is_superuser', False)
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, password, **extra_fields):
        """Crée et enregistre un compte super-utilisateur."""
        extra_fields.setdefault('is_superuser', True)
        if extra_fields.get('is_superuser') is not True:  # pragma: no cover
            raise ValueError(
                "Un super-utilisateur doit avoir is_superuser=True."
            )
        return self.create_user(email, password, **extra_fields)


class User(AbstractBaseUser, PermissionsMixin):
    """
    Un compte utilisateur, associé à une personne et utilisant l'adresse
    mail pour l'authentification
    """

    # Identité
    first_name = models.CharField("prénom", max_length=30, blank=True)
    last_name = models.CharField("nom", max_length=50, blank=True)
    pseudo = models.CharField("pseudo", max_length=30, blank=True)
    email = models.EmailField(
        "adresse mail",
        unique=True,
        error_messages={
            'unique': "Un compte avec cette adresse mail existe déjà."
        },
    )

    # Permissions
    is_active = models.BooleanField(
        "actif",
        default=True,
        help_text=(
            "Détermine si le compte utilisateur doit être considéré comme "
            "actif dans l'application. Vous pouvez le désactiver plutôt que "
            "de le supprimer."
        ),
    )

    # Méta-données
    date_joined = models.DateTimeField(
        "date d'inscription", default=timezone.now, editable=False
    )

    # Avatar
    def upload_avatar(instance, filename):
        extension = os.path.splitext(filename)[1]
        return 'avatars/{0}-{1}{2}'.format(
            uuid4(), slugify(str(instance)), extension
        )

    avatar = models.ImageField(
        "avatar", upload_to=upload_avatar, blank=True, null=True
    )
    avatar_thumb = models.ImageField(
        upload_to=upload_avatar, editable=False, blank=True, null=True
    )

    @property
    def pseudo_rand_color(self):
        return self.date_joined.microsecond % 360

    objects = UserManager()

    EMAIL_FIELD = 'email'
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    class Meta:
        verbose_name = "compte utilisateur"
        verbose_name_plural = "comptes utilisateurs"
        ordering = ('last_name', 'first_name', 'pseudo')

    @property
    def is_staff(self):
        # Détermine si le compte a accès à l'administration de Django, qu'on
        # vise à ne pas utiliser dans l'application.
        return self.is_superuser

    def __str__(self):
        return self.full_name or self.email

    def clean(self):
        """Vérifie les champs d'identité et normalise l'email."""
        super().clean()

        if not any([self.first_name, self.last_name, self.pseudo]):
            raise ValidationError(
                "Au moins le prénom, le nom ou le pseudo est requis.",
                code='name_required',
            )

        self.email = self.__class__.objects.normalize_email(self.email)

    def get_full_name(self):
        """Retourne le prénom et le nom de la personne, et/ou son pseudo."""
        if not self.first_name and not self.last_name:
            return self.pseudo
        full_name = "{} {}".format(self.first_name, self.last_name).strip()
        if self.pseudo:
            # ajoute le pseudo entre parenthèses
            full_name += " ({})".format(self.pseudo)
        return full_name

    full_name = property(get_full_name)

    def get_short_name(self):
        """Retourne le prénom, le nom ou le pseudo de la personne."""
        return self.first_name or self.last_name or self.pseudo

    short_name = property(get_short_name)

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Envoi un courriel à cette personne."""
        send_mail(subject, message, from_email, [self.email], **kwargs)

    def generate_avatar_thumbnail(self):
        if self.avatar_thumb:
            self.avatar_thumb.delete(False)
        if not self.avatar:
            return
        self.avatar_thumb = resize_image_to_file(
            self.avatar, AVATAR_THUMB_SIZE
        )

    def save(self, *args, **kwargs):
        self.generate_avatar_thumbnail()
        super().save(*args, **kwargs)
