from django.contrib import messages
from django.contrib.auth import views as auth_views
from django.contrib.messages.views import SuccessMessageMixin
from django.http import Http404, HttpResponseRedirect
from django.urls import reverse_lazy
from django.utils.html import format_html
from django.views.generic.base import TemplateView
from django.views.generic.edit import UpdateView

from registration.backends.default import views as registration_views

from benevalibre.utils.views import PageMixin

from .forms import (
    AuthenticationForm,
    PasswordResetForm,
    RegistrationForm,
    ResendActivationForm,
    SetPasswordForm,
    UserUpdateForm,
)
from .models import User

LOGIN_URL = reverse_lazy('accounts:login')


# AUTHENTICATION AND PASSWORD RESET
# ------------------------------------------------------------------------------


class LoginView(PageMixin, auth_views.LoginView):
    form_class = AuthenticationForm
    template_name = 'accounts/login.html'
    page_title = "Connexion"


class LogoutView(auth_views.LogoutView):
    def dispatch(self, request, *args, **kwargs):
        response = super().dispatch(request, *args, **kwargs)
        messages.success(request, "Vous avez été déconnecté.")
        return response


class PasswordResetView(PageMixin, auth_views.PasswordResetView):
    form_class = PasswordResetForm
    success_url = LOGIN_URL
    template_name = 'accounts/password_reset.html'
    email_template_name = 'accounts/password_reset_email.txt'
    html_email_template_name = 'accounts/password_reset_email.html'
    subject_template_name = 'accounts/password_reset_subject.txt'
    page_title = "Réinitialiser mon mot de passe"

    def form_valid(self, form):
        messages.success(
            self.request,
            format_html(
                (
                    "Un courriel a été envoyé à <i>{}</i>, si cette adresse "
                    "mail est valide. Il contient les instructions à suivre "
                    "pour réinitialiser votre mot de passe."
                ),
                form.cleaned_data['email'],
            ),
        )
        return super().form_valid(form)


class PasswordResetConfirmView(PageMixin, auth_views.PasswordResetConfirmView):
    form_class = SetPasswordForm
    success_url = LOGIN_URL
    template_name = 'accounts/password_reset_confirm.html'
    page_title = "Réinitialiser mon mot de passe"

    def get_success_url(self):
        messages.success(
            self.request, "Votre mot de passe a été réinitialisé."
        )
        return super().get_success_url()


# USER PROFILE MANAGEMENT
# ------------------------------------------------------------------------------


class UserUpdateView(PageMixin, SuccessMessageMixin, UpdateView):
    model = User
    form_class = UserUpdateForm
    page_title = "Mettre à jour son profil"
    success_message = "Votre profil a bien été modifié."
    template_name = 'accounts/update_user.html'
    success_url = reverse_lazy('board:home')

    def setup(self, *args, **kwargs):
        super().setup(*args, **kwargs)
        if kwargs['pk'] != self.request.user.pk:
            raise Http404


# REGISTRATION
# ------------------------------------------------------------------------------


class RegistrationView(PageMixin, registration_views.RegistrationView):
    form_class = RegistrationForm
    disallowed_url = reverse_lazy('accounts:registration_closed')
    success_url = reverse_lazy('accounts:registration_complete')
    page_title = "Créer un compte"


class RegistrationCompleteView(PageMixin, TemplateView):
    template_name = 'registration/registration_complete.html'
    page_title = "Créer un compte"


class RegistrationClosedView(PageMixin, TemplateView):
    template_name = 'registration/registration_closed.html'
    page_title = "Inscription indisponible"


class ActivationView(registration_views.ActivationView):
    def get_success_url(self, user):
        messages.success(
            self.request,
            format_html(
                (
                    "Votre compte est désormais activé ! Vous pouvez "
                    "maintenant vous connecter avec votre adresse <i>{}</i>."
                ),
                user.email,
            ),
        )
        return LOGIN_URL


class ResendActivationView(PageMixin, registration_views.ResendActivationView):
    form_class = ResendActivationForm
    page_title = "Renvoyer le courriel d'activation"

    def render_form_submitted_template(self, form):
        messages.success(
            self.request,
            format_html(
                (
                    "Un courriel a été envoyé à <i>{}</i>, si cette adresse "
                    "mail est valide. Il contient un nouveau lien permettant "
                    "d'activer votre compte."
                ),
                form.cleaned_data['email'],
            ),
        )
        return HttpResponseRedirect(LOGIN_URL)
