from django.contrib.messages import get_messages
from django.urls import reverse

import pytest
from bs4 import BeautifulSoup as bs

from benevalibre.utils.tests import count_text_in_content


@pytest.mark.django_db
class TestAssociation:
    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, "<!--Please Login-->"),
            ('user', "<!--Please Login-->"),
            ('staff', "Liste des associations"),
        ],
    )
    def test_get_index(self, client, user, superuser, use, expected):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(reverse('association:index'))
        assert response.status_code == 200
        assert count_text_in_content(response, expected)

    def test_post_create_update_delete(self, client, user, superuser):
        # 0. on fetch
        client.force_login(superuser)
        response = client.get(reverse('association:index'))

        # 1. on créé
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        with open('benevalibre/utils/tests/images/flower.jpg', 'rb') as logo:
            data = {'name': 'asso de test', 'logo': logo}
            response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "asso de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« asso de test » a été ajouté" in messages[0]

        # 2. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        with open('benevalibre/utils/tests/images/flower.jpg', 'rb') as logo:
            data = {'name': 'asso de lutte', 'logo': logo}
            response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert not count_text_in_content(response, "test")
        assert count_text_in_content(response, "asso de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        # 3. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find('tr')
        )
        assert len(table_row.contents) == 1
        assert len(table_row.contents[0].contents) == 1
        assert "Aucune donnée disponible." == table_row.contents[0].contents[0]
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]


@pytest.mark.django_db
class TestProjet:
    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, "<!--Please Login-->"),
            ('user', "<!--Please Login-->"),
            ('staff', "Liste des"),
        ],
    )
    def test_get_index(
        self, association, client, user, superuser, use, expected
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('association:project:index', args=[association.id])
        )
        assert response.status_code == 200
        assert count_text_in_content(response, expected)

    def test_post_create_update_delete(self, client, user, superuser):
        # 0. on fetch les assos
        client.force_login(superuser)
        response = client.get(reverse('association:index'))

        # 1. on créé une asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'asso de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "asso de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« asso de test » a été ajouté" in messages[0]

        # 2. on fetch les projets de l'asso
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find('a', class_='project-link')
            .attrs['href']
        )
        response = client.get(url)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find('tr')
        )
        assert len(table_row.contents) == 1
        assert len(table_row.contents[0].contents) == 1
        assert "Aucune donnée disponible." == table_row.contents[0].contents[0]

        # 3. on créé un projet dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'projet de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "projet de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« projet de test » a été ajouté" in messages[0]

        # 4. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'projet de lutte'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert not count_text_in_content(response, "projet de test")
        assert count_text_in_content(response, "projet de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        # 5. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find('tr')
        )
        assert len(table_row.contents) == 1
        assert len(table_row.contents[0].contents) == 1
        assert "Aucune donnée disponible." == table_row.contents[0].contents[0]
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]


@pytest.mark.django_db
class TestEngagement:
    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, "<!--Please Login-->"),
            ('user', "<!--Please Login-->"),
            ('staff', "Liste des"),
        ],
    )
    def test_get_index(
        self, association, client, user, superuser, use, expected
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('association:engagement:index', args=[association.id])
        )
        assert response.status_code == 200
        assert count_text_in_content(response, expected)

    def test_post_create_update_delete(self, client, user, role, superuser):
        # 0. on fetch les assos
        client.force_login(superuser)
        response = client.get(reverse('association:index'))

        # 1. on créé une asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'asso de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "asso de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« asso de test » a été ajouté" in messages[0]

        # 2. on fetch les engagements de l'asso
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find('a', class_='engagement-link')
            .attrs['href']
        )
        response = client.get(url)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find('tr')
        )
        assert len(table_row.contents) == 1
        assert len(table_row.contents[0].contents) == 1
        assert "Aucune donnée disponible." == table_row.contents[0].contents[0]

        # 3. on créé un engagement dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'user': user.id, 'role': role.id}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "RoleTest")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« user@example.org est RoleTest » a été ajouté" in messages[0]

        # 4. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        role = role.association.role_set.get_dirigeant()
        data = {'user': user.id, 'role': role.id}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert not count_text_in_content(response, "RoleTest")
        assert count_text_in_content(response, "Dirigeant")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert (
            "« user@example.org est Dirigeant » a été modifié" in messages[0]
        )

        # 5. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find('tr')
        )
        assert len(table_row.contents) == 1
        assert len(table_row.contents[0].contents) == 1
        assert "Aucune donnée disponible." == table_row.contents[0].contents[0]
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]


@pytest.mark.django_db
class TestLevel:
    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, "<!--Please Login-->"),
            ('user', "<!--Please Login-->"),
            ('staff', "Liste des"),
        ],
    )
    def test_get_index(
        self, association, client, user, superuser, use, expected
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('association:level:index', args=[association.id])
        )
        assert response.status_code == 200
        assert count_text_in_content(response, expected)

    def test_post_create_update_delete(self, client, user, superuser):
        # 0. on fetch les assos
        client.force_login(superuser)
        response = client.get(reverse('association:index'))

        # 1. on créé une asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'asso de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "asso de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« asso de test » a été ajouté" in messages[0]

        # 2. on fetch les levels de l'asso
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find('a', class_='level-link')
            .attrs['href']
        )
        response = client.get(url)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find('tr')
        )
        assert len(table_row.contents) == 1
        assert len(table_row.contents[0].contents) == 1
        assert "Aucune donnée disponible." == table_row.contents[0].contents[0]

        # 3. on créé un level dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'level de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "level de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« level de test » a été ajouté" in messages[0]

        # 4. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'level de lutte'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert not count_text_in_content(response, "level de test")
        assert count_text_in_content(response, "level de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        # 5. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find('tr')
        )
        assert len(table_row.contents) == 1
        assert len(table_row.contents[0].contents) == 1
        assert "Aucune donnée disponible." == table_row.contents[0].contents[0]
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]


@pytest.mark.django_db
class TestRole:
    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, "<!--Please Login-->"),
            ('user', "<!--Please Login-->"),
            ('staff', "Liste des"),
        ],
    )
    def test_get_index(
        self, association, client, user, superuser, use, expected
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('association:role:index', args=[association.id])
        )
        assert response.status_code == 200
        assert count_text_in_content(response, expected)

    def test_post_create_update_delete(self, client, user, superuser):
        # 0. on fetch les assos
        client.force_login(superuser)
        response = client.get(reverse('association:index'))

        # 1. on créé une asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'asso de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "asso de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« asso de test » a été ajouté" in messages[0]

        # 2. on fetch les roles de l'asso
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find('a', class_='role-link')
            .attrs['href']
        )
        response = client.get(url)
        assert response.status_code == 200
        default_roles = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(default_roles) == 3

        # 3. on créé un role dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'role de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "role de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« role de test » a été ajouté" in messages[0]
        new_roles = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(default_roles) + 1 == len(new_roles)

        # 4. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'role de lutte'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert not count_text_in_content(response, "role de test")
        assert count_text_in_content(response, "role de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        # 5. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]
        new_roles = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(default_roles) == len(new_roles)


@pytest.mark.django_db
class TestCategories:
    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, "<!--Please Login-->"),
            ('user', "<!--Please Login-->"),
            ('staff', "Liste des"),
        ],
    )
    def test_get_index(
        self, association, client, user, superuser, use, expected
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('association:category:index', args=[association.id])
        )
        assert response.status_code == 200
        assert count_text_in_content(response, expected)

    def test_post_create_update_delete(self, client, user, superuser):

        client.force_login(superuser)

        # 0. on créé une categorie d'instance
        url = reverse('instance:instance_category:create')
        data = {'name': 'instant cat test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "instant cat test", html=True)
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« instant cat test » a été ajouté" in messages[0]

        # 1. on fetch les assos
        response = client.get(reverse('association:index'))

        # 2. on créé une asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'asso de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "asso de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« asso de test » a été ajouté" in messages[0]

        # 3. on fetch les categorys de l'asso
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find('a', class_='category-link')
            .attrs['href']
        )
        response = client.get(url)
        assert response.status_code == 200
        default_categories = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )

        # 4. on créé une categorie dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'categorie de test', 'instance_category': 1}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "categorie de test")
        new_categories = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(default_categories) + 1 == len(new_categories)
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« categorie de test » a été ajouté" in messages[0]

        # 5. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'categorie de lutte', 'instance_category': 1}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert not count_text_in_content(response, "categorie de test")
        assert count_text_in_content(response, "categorie de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« categorie de lutte » a été modifié" in messages[0]

        # 6. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« categorie de lutte » a été supprimé" in messages[0]
