from django.shortcuts import get_object_or_404
from django.urls import reverse, reverse_lazy

from cruditor import views

from benevalibre.utils.html import fa_icon
from benevalibre.utils.views import StaffCruditorMixin

from . import forms, models, tables


class AssociatedViewMixin:
    def get_title(self):
        return "{} pour « {} »".format(super().get_title(), self.association)

    def setup(self, request, *args, **kwargs):
        super().setup(request, *args, **kwargs)
        self.association = self.get_association()

    def get_association(self):
        return get_object_or_404(
            models.Association.objects.viewable_by(
                self.request.user
            ).distinct(),
            pk=self.kwargs['association'],
        )


class AssociatedEditionMixin:
    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.association = self.association
        return super().form_valid(form)


class AssociationViewMixin(StaffCruditorMixin):
    model = models.Association
    success_url = reverse_lazy('association:index')


class Index(AssociationViewMixin, views.CruditorListView):
    title = "Liste des associations"
    table_class = tables.AssociationIndexTable

    def get_titlebutton_condition(self):
        return True

    def get_titlebutton_url(self):
        return reverse('association:create')

    def get_titlebutton_label(self):
        return fa_icon('plus-circle', "Ajouter une association")


class Create(AssociationViewMixin, views.CruditorAddView):
    form_class = forms.CreateForm


class Update(AssociationViewMixin, views.CruditorChangeView):
    form_class = forms.UpdateForm

    def get_delete_url(self):
        return reverse('association:delete', args=(self.object.pk,))


class Delete(AssociationViewMixin, views.CruditorDeleteView):
    pass


class ProjectViewMixin(AssociatedViewMixin, StaffCruditorMixin):
    model = models.Project

    def get_success_url(self):
        return reverse(
            'association:project:index', args=[self.kwargs['association']]
        )


class ProjectIndex(ProjectViewMixin, views.CruditorListView):
    title = "Liste des projets de l'association"
    table_class = tables.ProjectIndexTable

    def get_queryset(self):
        return self.association.project_set.all()

    def get_titlebutton_condition(self):
        return True

    def get_titlebutton_url(self):
        return reverse(
            'association:project:create', args=[self.association.id]
        )

    def get_titlebutton_label(self):
        return fa_icon('plus-circle', "Ajouter un projet")


class ProjectCreate(
    ProjectViewMixin, AssociatedEditionMixin, views.CruditorAddView
):
    form_class = forms.ProjectCreateForm


class ProjectUpdate(
    ProjectViewMixin, AssociatedEditionMixin, views.CruditorChangeView
):
    form_class = forms.ProjectUpdateForm

    def get_delete_url(self):
        return reverse(
            'association:project:delete',
            args=(self.kwargs['association'], self.object.pk),
        )


class ProjectDelete(ProjectViewMixin, views.CruditorDeleteView):
    pass


class RoleViewMixin(AssociatedViewMixin, StaffCruditorMixin):
    model = models.Role

    def get_success_url(self):
        return reverse(
            'association:role:index', args=[self.kwargs['association']]
        )


class RoleIndex(RoleViewMixin, views.CruditorListView):
    title = "Liste des rôles"
    table_class = tables.RoleIndexTable

    def get_queryset(self):
        return self.association.role_set.all()

    def get_titlebutton_condition(self):
        return True

    def get_titlebutton_url(self):
        return reverse('association:role:create', args=[self.association.id])

    def get_titlebutton_label(self):
        return fa_icon('plus-circle', "Ajouter un rôle")


class RoleCreate(RoleViewMixin, AssociatedEditionMixin, views.CruditorAddView):
    form_class = forms.RoleCreateForm


class RoleUpdate(
    RoleViewMixin, AssociatedEditionMixin, views.CruditorChangeView
):
    form_class = forms.RoleUpdateForm

    def get_delete_url(self):
        return reverse(
            'association:role:delete',
            args=(self.kwargs['association'], self.object.pk),
        )


class RoleDelete(RoleViewMixin, views.CruditorDeleteView):
    pass


class EngagementViewMixin(AssociatedViewMixin, StaffCruditorMixin):
    model = models.Engagement

    def get_success_url(self):
        return reverse(
            'association:engagement:index', args=[self.kwargs['association']]
        )


class EngagementIndex(EngagementViewMixin, views.CruditorListView):
    title = "Liste des bénévoles"
    table_class = tables.EngagementIndexTable

    def get_queryset(self):
        return self.association.engagement_set.all()

    def get_titlebutton_condition(self):
        return True

    def get_titlebutton_url(self):
        return reverse(
            'association:engagement:create', args=[self.association.id]
        )

    def get_titlebutton_label(self):
        return fa_icon('plus-circle', "Ajouter un bénévole")


class EngagementCreate(
    EngagementViewMixin, AssociatedEditionMixin, views.CruditorAddView
):
    form_class = forms.EngagementCreateForm

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['association'] = self.association
        return kwargs


class EngagementUpdate(
    EngagementViewMixin, AssociatedEditionMixin, views.CruditorChangeView
):
    form_class = forms.EngagementUpdateForm

    def get_delete_url(self):
        return reverse(
            'association:engagement:delete',
            args=(self.kwargs['association'], self.object.pk),
        )

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['association'] = self.association
        return kwargs


class EngagementDelete(EngagementViewMixin, views.CruditorDeleteView):
    pass


class LevelViewMixin(AssociatedViewMixin, StaffCruditorMixin):
    model = models.Level

    def get_success_url(self):
        return reverse(
            'association:level:index', args=[self.kwargs['association']]
        )


class LevelIndex(LevelViewMixin, views.CruditorListView):
    title = "Liste des niveaux de bénévolat"
    table_class = tables.LevelIndexTable

    def get_queryset(self):
        return self.association.level_set.all()

    def get_titlebutton_condition(self):
        return True

    def get_titlebutton_url(self):
        return reverse('association:level:create', args=[self.association.id])

    def get_titlebutton_label(self):
        return fa_icon('plus-circle', "Ajouter un niveau de bénévolat")


class LevelCreate(
    LevelViewMixin, AssociatedEditionMixin, views.CruditorAddView
):
    form_class = forms.LevelCreateForm


class LevelUpdate(
    LevelViewMixin, AssociatedEditionMixin, views.CruditorChangeView
):
    form_class = forms.LevelUpdateForm

    def get_delete_url(self):
        return reverse(
            'association:level:delete',
            args=(self.kwargs['association'], self.object.pk),
        )


class LevelDelete(LevelViewMixin, views.CruditorDeleteView):
    pass


class CategoryViewMixin(AssociatedViewMixin, StaffCruditorMixin):
    model = models.Category

    def get_success_url(self):
        return reverse(
            'association:category:index', args=[self.kwargs['association']]
        )


class CategoryIndex(CategoryViewMixin, views.CruditorListView):
    title = "Liste des catégories de bénévolat"
    table_class = tables.CategoryIndexTable

    def get_queryset(self):
        return self.association.category_set.all()

    def get_titlebutton_condition(self):
        return True

    def get_titlebutton_url(self):
        return reverse(
            'association:category:create', args=[self.association.id]
        )

    def get_titlebutton_label(self):
        return fa_icon('plus-circle', "Ajouter une catégorie")


class CategoryCreate(
    CategoryViewMixin, AssociatedEditionMixin, views.CruditorAddView
):
    form_class = forms.CategoryCreateForm


class CategoryUpdate(
    CategoryViewMixin, AssociatedEditionMixin, views.CruditorChangeView
):
    form_class = forms.CategoryUpdateForm

    def get_delete_url(self):
        return reverse(
            'association:category:delete',
            args=(self.kwargs['association'], self.object.pk),
        )


class CategoryDelete(CategoryViewMixin, views.CruditorDeleteView):
    pass
