from datetime import date, timedelta

from django.contrib.messages import get_messages
from django.urls import reverse

import pytest
from bs4 import BeautifulSoup as bs

from benevalibre.utils.tests import count_text_in_content


@pytest.mark.django_db
class TestLandingPage:
    def test_landing_url(self, client):
        response = client.get('')
        assert response.status_code == 302
        assert response.url == reverse('board:home')

    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, 'board:association:index'),
            ('user', 'board:board'),
            ('staff', 'board:association:index'),
        ],
    )
    def test_get_index(
        self, client, association, engagement, user, superuser, use, expected
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(reverse('board:home'))
        assert response.status_code == 302
        assert response.url == reverse(expected)

    def test_description_in_table(self, client, association):
        response = client.get('', follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "des associations, une …")


@pytest.mark.django_db
class TestIndex:
    @pytest.mark.parametrize(
        'use, expected', [(None, 1), ('user', 2), ('staff', 3)]
    )
    def test_get_index(
        self, client, association, engagement, user, superuser, use, expected
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        """Viewable only by staff"""
        association._meta.model.objects.create(name='bling', conceal=True)

        """Viewable only by staff and user"""
        asso = association._meta.model.objects.create(
            name='blang', conceal=True
        )
        asso.engagement_set.create(user=user, role=asso.role_set.last())

        response = client.get(reverse('board:association:index'))
        assert response.status_code == 200
        assert count_text_in_content(response, "Bienvenue")
        assert count_text_in_content(response, "AssoTest")
        assert count_text_in_content(response, "Enregistrer une association")

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == expected

    def test_get_index_registration_closed(self, client, closed_registration):
        response = client.get(reverse('board:association:index'))
        assert response.status_code == 200
        assert not count_text_in_content(
            response, "Enregistrer une association"
        )

    def test_create_registration_closed(self, client, closed_registration):
        response = client.get(reverse('board:association:create'))
        assert response.status_code == 404

    def test_association_create_update_delete(self, client, user):
        client.force_login(user)

        # 0. on fetch le board
        response = client.get('', follow=True)

        # 1. on créé
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )

        data = {'name': 'asso de test'}
        response = client.post(url, data, follow=True)

        assert response.status_code == 200
        assert count_text_in_content(response, "asso de test")
        assert count_text_in_content(response, "Secteur d'activité")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été ajouté" in messages[0]

        assert (
            user.engagement_set.active()
            .filter(role__manage_association=True)
            .exists()
        )

        buttons = (
            bs(response.content, 'html.parser')
            .find(id='main-menu')
            .find_all('a', class_='dropdown-item capabilities')
        )
        assert len(buttons) == 7

        # 2. on update
        url = [b.attrs['href'] for b in buttons if "Modifier" in str(b)][0]
        data = {'name': 'asso de test', 'description': 'Une asso qui en jette'}
        response = client.post(url, data, follow=True)

        assert response.status_code == 200
        assert count_text_in_content(response, "qui en jette")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        # 3. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

    def test_association_create_twice(self, client, association, user):
        client.force_login(user)

        # 0. on fetch le board
        response = client.get('', follow=True)

        # 1. on tente de créer
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )

        data = {'name': 'AssoTest'}
        response = client.post(url, data, follow=True)

        assert response.status_code == 200
        assert count_text_in_content(response, "Ajouter association")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "Une association existe déjà sous ce nom." in messages[0]

        # 2. on force la création
        data = {'name': 'AssoTest'}
        response = client.post(url, data, follow=True)

        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été ajouté" in messages[0]

    def test_moderated_association_create(
        self, client, user, foreign, superuser, moderated_registration
    ):
        assert not user.is_superuser
        client.force_login(user)

        # 1. On demande la création
        response = client.get('', follow=True)

        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )

        data = {'name': 'asso de test'}
        response = client.post(url, data, follow=True)

        assert response.status_code == 200
        assert count_text_in_content(response, "Mes engagements associatifs")
        assert not count_text_in_content(response, "Secteur d'activité")
        assert "asso de test" not in bs(response.content, 'html.parser').find(
            'tbody'
        ).find_all('tr')

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "modération" in messages[0]

        # 2. On vérifie qu'on a accès
        association = user.engagement_set.active().last().association
        assert association.name == "asso de test"
        assert not association.is_active

        response = client.get(
            reverse('board:association:detail', args=[association.id])
        )
        assert response.status_code == 200

        # 3. On vérifie qu'un autre n'y a pas accès

        client.logout()
        client.force_login(foreign)
        response = client.get(
            reverse('board:association:detail', args=[association.id])
        )
        assert response.status_code == 404

        # 3. Elle est visible à un superuser

        client.logout()
        client.force_login(superuser)
        response = client.get(reverse('board:board'), follow=True)
        assert count_text_in_content(response, "Modérations d'associations")

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "asso de test" in str(table_row[0])
        assert "user@example.org" in str(table_row[0])
        assert "(en modération)" in str(table_row[0])

        # 4. Un superuser donne suite à la demande
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        response = client.get(url, follow=True)

        # On prend tout et on post
        form = bs(response.content, 'html.parser').find("form")
        input_kv = [
            (i.get('name'), i.get('value'))
            for i in form.find_all('input')
            if i.get('value')
        ]
        select_kv = [
            (s.parent.get('name'), s.get('value'))
            for s in form.find_all('option', attrs={"selected": ""})
            if s.get('value')
        ]
        data = dict(input_kv + select_kv)

        response = client.post(url, data, follow=True)

        response = client.get('', follow=True)
        assert not count_text_in_content(
            response, "Modérations d'associations"
        )

        # 5. On a désormais accès
        assert count_text_in_content(response, "Bienvenue à bord")

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "asso de test" in str(table_row[0])
        assert "(en  modération)" not in str(table_row[0])
        assert "disabled" not in str(table_row[0])

        # 6. Le bénévole a désormais accès
        client.logout()
        client.force_login(user)
        response = client.get('', follow=True)

        assert count_text_in_content(response, "Mes engagements associatifs")

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "asso de test" in str(table_row[0])
        assert "Dirigeant" in str(table_row[0])
        assert "(en  modération)" not in str(table_row[0])
        assert "disabled" not in str(table_row[0])

    def test_get_on_association_engagement_go_back_to_home(
        self, client, association, user
    ):
        client.force_login(user)
        response = client.get(
            reverse('board:association:engage', args=[association.id])
        )
        assert response.url == reverse('board:association:index')

    def test_association_engagement(self, client, user, association):
        client.force_login(user)
        response = client.get('', follow=True)

        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find('form')
            .attrs['action']
        )

        data = {}
        response = client.post(url, data, follow=True)

        assert response.status_code == 200
        assert (
            user.engagement_set.active()
            .filter(association=association)
            .exists()
        )
        assert count_text_in_content(response, "Mes engagements associatifs")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "modération" not in messages[0]
        assert "acceptée" in messages[0]

    def test_association_engagement_twice(self, client, user, association):
        client.force_login(user)
        response = client.get('', follow=True)

        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find('form')
            .attrs['action']
        )

        data = {}
        response = client.post(url, data, follow=True)
        response = client.post(url, data, follow=True)

        assert response.status_code == 200
        assert count_text_in_content(response, "Bienvenue à bord")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "déjà engagé dans cette association" in messages[0]

    def test_moderated_association_engagement(
        self, client, user, association, engagement, foreign
    ):
        association.moderate_engagement = True
        association.save()
        engagement.role = association.role_set.get_dirigeant()
        engagement.save()

        # 1. On fait la demande d'engagement
        client.force_login(foreign)
        response = client.get('', follow=True)

        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find('form')
            .attrs['action']
        )

        data = {}
        response = client.post(url, data, follow=True)

        # 2. Elle est suspendue en modération
        assert response.status_code == 200
        assert foreign.engagement_set.filter(association=association).exists()
        assert (
            not foreign.engagement_set.active()
            .filter(association=association)
            .exists()
        )
        assert count_text_in_content(response, "Mes engagements associatifs")
        assert count_text_in_content(response, "Enregistrer une association")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "modération" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "AssoTest" in str(table_row[0])
        assert 'href="#"' in str(table_row[0])
        assert "disabled" in str(table_row[0])
        assert "En modération" in str(table_row[0])

        # 3. Elle est visible à un modérateur
        client.logout()
        client.force_login(user)
        response = client.get('', follow=True)
        assert count_text_in_content(response, "Modérations de bénévoles")

        # 4. On donne suite
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'role': association.role_set.get_benevole().id}
        response = client.post(url, data, follow=True)
        response = client.get('', follow=True)
        assert not count_text_in_content(response, "Modérations de bénévoles")

        # 5. Le bénévole a désormais accès
        client.logout()
        client.force_login(foreign)
        response = client.get('', follow=True)

        assert count_text_in_content(response, "Mes engagements associatifs")
        assert count_text_in_content(response, "Enregistrer une association")

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "AssoTest" in str(table_row[0])
        assert "disabled" not in str(table_row[0])


@pytest.mark.django_db
class TestBoard:
    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, "<!--Please Login-->"),
            ('user', "AssoTest"),
            ('staff', "Aucune donnée disponible"),
        ],
    )
    def test_get_engagement(
        self, client, association, engagement, user, superuser, use, expected
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        """Engaged by no one"""
        association._meta.model.objects.create(name='bling', conceal=True)
        association._meta.model.objects.create(name='blang')

        response = client.get(reverse('board:board'))
        assert response.status_code == 200

        assert count_text_in_content(response, expected)

        if use:
            assert count_text_in_content(
                response, "Mes engagements associatifs"
            )
            assert count_text_in_content(
                response, "Enregistrer une association"
            )

    @pytest.mark.parametrize(
        'use, expected', [(None, 404), ('user', 200), ('staff', 404)]
    )
    def test_create_associated_self_benevalo_access(
        self, client, association, engagement, user, superuser, use, expected
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse(
                'board:association:self-benevalo:create', args=[association.id]
            )
        )
        assert response.status_code == expected

    def test_create_update_delete_associated_self_benevalo(
        self, client, association, category, engagement, user, superuser
    ):
        client.force_login(user)

        # 0. On fetch les engagements
        response = client.get(reverse('board:board'))
        assert response.status_code == 200

        # 1. on créé
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='create_benevalo-link')[-1]
            .attrs['href']
        )
        response = client.get(url, follow=True)
        assert not count_text_in_content(response, "level")
        assert not count_text_in_content(response, "project")

        data = {
            'user': user.id,
            'title': 'bénévalo de test',
            'duration': '33',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        association.benevalo_set.get(user=user, title='bénévalo de test')

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été ajouté" in messages[0]

        # 2. On a la liste

        table = str(bs(response.content, 'html.parser').find('tbody'))

        assert "33" in table
        assert "bénévalo de test" in table

        # 3. On update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {
            'user': user.id,
            'title': 'bénévalo de lutte',
            'duration': '42',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }
        response = client.post(url, data, follow=True)

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table = str(bs(response.content, 'html.parser').find('tbody'))
        assert "33" not in table
        assert "bénévalo de test" not in table
        assert "42" in table
        assert "bénévalo de lutte" in table

        # 4. on vérifie la date dans le formulaire  (BUG #20)
        tmp_response = client.get(url, follow=True)
        execution_date = (
            bs(tmp_response.content, 'html.parser')
            .find(id='id_execution_date')
            .attrs['value']
        )
        assert execution_date == date.today().strftime('%Y-%m-%d')

        # 5. On delete
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='delete-link')[-1]
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "» a été supprimé" in messages[0]

        table = str(bs(response.content, 'html.parser').find('tbody'))
        assert "Aucune donnée disponible." in table

    def test_create_associated_benevalo_with_one_level(
        self, client, association, level, category, engagement, user, superuser
    ):
        client.force_login(user)

        # 0. On fetch les engagements
        response = client.get(reverse('board:board'))
        assert response.status_code == 200

        # 1. on créé
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='create_benevalo-link')[-1]
            .attrs['href']
        )
        response = client.get(url, follow=True)
        assert count_text_in_content(response, "level")
        assert not count_text_in_content(response, "project")

        data = {
            'user': user.id,
            'title': 'BenevaloTest',
            'duration': '33',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }

        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        association.benevalo_set.get(user=user, title="BenevaloTest")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été ajouté" in messages[0]

    def test_create_associated_benevalo_with_one_project(
        self,
        client,
        association,
        project,
        category,
        engagement,
        user,
        superuser,
    ):
        client.force_login(user)

        # 0. On fetch les engagements
        response = client.get(reverse('board:board'))
        assert response.status_code == 200

        # 1. on créé
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='create_benevalo-link')[-1]
            .attrs['href']
        )
        response = client.get(url, follow=True)
        assert count_text_in_content(response, "project")
        assert not count_text_in_content(response, "level")

        data = {
            'user': user.id,
            'title': 'test',
            'duration': '33',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }

        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        association.benevalo_set.get(user=user, title='test')

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été ajouté" in messages[0]

    def test_create_associated_benevalo_with_moderation(
        self, client, association, category, manager, user, foreign
    ):
        association.moderate_benevalo = True
        association.save()

        engagement = association.engagement_set.create(
            user=foreign, role=association.role_set.get_benevole()
        )

        # 0. On fetch les engagements
        client.force_login(foreign)
        response = client.get(reverse('board:board'))
        assert response.status_code == 200

        assert not count_text_in_content(response, "level")

        # 1. On fait la saisie
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='create_benevalo-link')[-1]
            .attrs['href']
        )
        data = {
            'user': user.id,
            'title': 'BenevaloTest',
            'duration': '33:00',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        benevalo = association.benevalo_set.get(
            user=foreign, title='BenevaloTest'
        )
        assert not benevalo.is_active

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "modération" in messages[0]

        # 2. Elle est visible mais suspendue en modération
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert "(en modération)" in str(table_row[0])

        # 3. Elle est visible à un modérateur
        client.logout()
        client.force_login(user)
        for urlname, args in [
            ('board:board', []),
            ('board:association:benevalo:index', [association.id]),
            (
                'board:association:engagement:benevalo',
                [association.id, engagement.id],
            ),
        ]:
            response = client.get(reverse(urlname, args=args))
            assert count_text_in_content(
                response, "Modérations de bénévolats en attente"
            )
            table_row = (
                bs(response.content, 'html.parser')
                .find('tbody')
                .find_all('tr')
            )
            assert len(table_row) == 1
            assert "foreign@example.org" in str(table_row[0])
            assert "BenevaloTest" in str(table_row[0])
            assert "Inspecter" in str(table_row[0])

        # 4. On donne suite
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        response = client.get(url, follow=True)

        # On prend tout et on post
        form = bs(response.content, 'html.parser').find("form")
        input_kv = [
            (i.get('name'), i.get('value'))
            for i in form.find_all('input')
            if i.get('value')
        ]
        select_kv = [
            (s.parent.get('name'), s.get('value'))
            for s in form.find_all('option', attrs={"selected": ""})
            if s.get('value')
        ]
        data = dict(input_kv + select_kv)

        response = client.post(url, data, follow=True)

        assert not count_text_in_content(
            response, "Modérations de bénévolats en attente"
        )

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        # 5. Le bénévole a désormais vue sur une saisie normale
        client.logout()
        client.force_login(foreign)
        response = client.get(
            reverse(
                'board:association:self-benevalo:index', args=[association.id]
            )
        )
        assert response.status_code == 200

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert "(en modération)" not in str(table_row[0])
        assert "33" in str(table_row[0])
        assert "BenevaloTest" in str(table_row[0])


@pytest.mark.django_db
class TestAssoDetails:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 200), ('user', 200), ('staff', 200), ('foreign', 200)],
    )
    def test_access_association_details(
        self, client, association, user, superuser, foreign, use, expected
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('board:association:detail', args=[association.id])
        )
        assert response.status_code == expected
        assert count_text_in_content(response, "association loi de 1901")

    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 404), ('staff', 200), ('foreign', 404)],
    )
    def test_access_conceal_association_details(
        self, client, association, user, superuser, foreign, use, expected
    ):
        association.conceal = True
        association.save()

        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('board:association:detail', args=[association.id])
        )
        assert response.status_code == expected

    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 200), ('staff', 200), ('foreign', 404)],
    )
    def test_access_conceal_association_details_with_engagement(
        self,
        client,
        association,
        engagement,
        user,
        superuser,
        foreign,
        use,
        expected,
    ):
        association.conceal = True
        association.save()

        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('board:association:detail', args=[association.id])
        )
        assert response.status_code == expected

    @pytest.mark.parametrize(
        'use, expected', [(None, 0), ('user', 7), ('staff', 1), ('foreign', 0)]
    )
    def test_association_details_management_buttons(
        self,
        client,
        association,
        user,
        superuser,
        foreign,
        manager,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('board:association:detail', args=[association.id])
        )
        assert response.status_code == 200

        buttons = (
            bs(response.content, 'html.parser')
            .find(id='main-menu')
            .find_all('a', class_='dropdown-item capabilities')
        )
        assert len(buttons) == expected

    @pytest.mark.parametrize(
        'userole,expected',
        [
            ('Bénévole', None),
            ('Bénévole++', 1),
            ('Animateur', 5),
            ('Dirigeant', 7),
        ],
    )
    def test_access_association_manage_features(
        self, client, association, user, engagement, userole, expected
    ):
        client.force_login(user)

        # munge role for benevole ++
        if userole == 'Bénévole++':
            userole = 'Bénévole'
            role = association.role_set.get_benevole()
            role.list_users = True
            role.save()

        # set up roles
        engagement = association.get_engagement(user)
        engagement.role = association.role_set.get(name=userole)
        engagement.save()

        # get the page
        response = client.get(
            reverse('board:association:detail', args=[association.id])
        )
        assert response.status_code == 200

        buttons = (
            bs(response.content, 'html.parser')
            .find(id='main-menu')
            .find_all('a', class_='dropdown-item capabilities')
        )
        assert bool(buttons) == bool(expected) or len(buttons) == expected

    @pytest.mark.parametrize(
        'use, expected',
        [(None, 200), ('user', 200), ('staff', 200), ('foreign', 404)],
    )
    def test_association_management_access(
        self,
        client,
        association,
        user,
        superuser,
        foreign,
        manager,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('board:association:update', args=[association.id])
        )
        assert response.status_code == expected
        if not use:
            assert count_text_in_content(response, "<!--Please Login-->")

    # Cruditor appelle get_object plus tôt dans le cas du delete et donc
    # ne retourne pas un formulaire de login
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 200), ('staff', 200), ('foreign', 404)],
    )
    def test_association_deletion_access(
        self,
        client,
        association,
        user,
        superuser,
        foreign,
        manager,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('board:association:delete', args=[association.id])
        )
        assert response.status_code == expected


@pytest.mark.django_db
class TestLevel:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 200), ('staff', 404), ('foreign', 404)],
    )
    def test_get_index(
        self,
        association,
        client,
        user,
        manager,
        superuser,
        foreign,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('board:association:level:index', args=[association.id])
        )
        assert response.status_code == expected

    def test_post_create_update_delete(
        self, client, association, manager, user
    ):
        client.force_login(user)

        # 1. on fetch les levels de l'asso
        url = reverse('board:association:level:index', args=[association.id])
        response = client.get(url)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Aucune donnée disponible." in str(table_row[-1])

        # 2. on créé un level dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'level de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "level de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« level de test » a été ajouté" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "level de test" in str(table_row[-1])

        # 3. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'level de lutte'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        assert not count_text_in_content(response, "level de test")
        assert count_text_in_content(response, "level de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "level de lutte" in str(table_row[-1])

        # 4. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Aucune donnée disponible." in str(table_row[-1])


@pytest.mark.django_db
class TestCategory:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 200), ('staff', 404), ('foreign', 404)],
    )
    def test_get_index(
        self,
        association,
        client,
        user,
        manager,
        superuser,
        foreign,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('board:association:category:index', args=[association.id])
        )
        assert response.status_code == expected

    def test_post_create_update_delete(
        self, client, association, manager, user
    ):
        client.force_login(user)

        # 1. on fetch les categories de l'asso
        url = reverse(
            'board:association:category:index', args=[association.id]
        )
        response = client.get(url)
        assert response.status_code == 200

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 12

        # 2. on créé une categorie dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'category de test', 'instance_category': 1}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "category de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« category de test » a été ajouté" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 13
        assert "category de test" in str(table_row[-1])

        # 3. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'category de lutte', 'instance_category': 1}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert not count_text_in_content(response, "category de test")
        assert count_text_in_content(response, "category de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 13
        assert "category de lutte" in str(table_row[-1])

        # 4. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 12


@pytest.mark.django_db
class TestProject:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 200), ('staff', 404), ('foreign', 404)],
    )
    def test_get_index(
        self,
        association,
        client,
        user,
        manager,
        superuser,
        foreign,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('board:association:project:index', args=[association.id])
        )
        assert response.status_code == expected

    def test_post_create_update_delete(
        self, client, association, manager, user
    ):
        client.force_login(user)

        # 1. on fetch les projects de l'asso
        url = reverse('board:association:project:index', args=[association.id])
        response = client.get(url)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Aucune donnée disponible." in str(table_row[-1])

        # 2. on créé un project dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'project de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "project de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« project de test » a été ajouté" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "project de test" in str(table_row[-1])

        # 3. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'project de lutte'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        assert not count_text_in_content(response, "project de test")
        assert count_text_in_content(response, "project de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "project de lutte" in str(table_row[-1])

        # 4. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Aucune donnée disponible." in str(table_row[-1])


@pytest.mark.django_db
class TestRole:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 200), ('staff', 404), ('foreign', 404)],
    )
    def test_get_index(
        self,
        association,
        client,
        user,
        manager,
        superuser,
        foreign,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('board:association:role:index', args=[association.id])
        )
        assert response.status_code == expected

    def test_post_create_update_delete(
        self, client, association, manager, user
    ):
        client.force_login(user)

        # 1. on fetch les roles de l'asso
        url = reverse('board:association:role:index', args=[association.id])
        response = client.get(url)
        assert response.status_code == 200

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 3

        # 2. on créé un role dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'role de test', 'instance_role': 1}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "role de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« role de test » a été ajouté" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 4
        assert "role de test" in str(table_row[-1])

        # 3. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'role de lutte', 'instance_role': 1}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert not count_text_in_content(response, "role de test")
        assert count_text_in_content(response, "role de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 4
        assert "role de lutte" in str(table_row[-1])

        # 4. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 3

    def test_self_erase_go_back_to_home(
        self, association, client, user, manager
    ):
        client.force_login(user)

        assert association.can_manage_roles(user)

        data = {'confirm': True}
        response = client.post(
            reverse(
                'board:association:role:delete',
                args=[association.id, manager.role.id],
            ),
            data,
            follow=True,
        )

        assert not association.can_manage_roles(user)

        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

        assert count_text_in_content(response, "Mes engagements associatifs")


@pytest.mark.django_db
class TestEngagement:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 200), ('staff', 404), ('foreign', 404)],
    )
    def test_get_index(
        self,
        association,
        client,
        user,
        manager,
        superuser,
        foreign,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse(
                'board:association:engagement:index', args=[association.id]
            )
        )
        assert response.status_code == expected

    def test_post_create_update_delete(
        self, client, association, manager, user, foreign
    ):
        client.force_login(user)

        # 1. on fetch les engagements de l'asso
        url = reverse(
            'board:association:engagement:index', args=[association.id]
        )
        response = client.get(url)
        assert response.status_code == 200

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Dirigeant" in str(table_row[-1])

        # 2. on update notre rôle
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'role': association.role_set.get(name="Animateur").id}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Animateur" in str(table_row[-1])

        # 3. on tente de restorer mais on n'est plus en capacité
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'role': association.role_set.get_dirigeant().id}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        form = bs(response.content, 'html.parser').find('form')
        assert "Ce choix ne fait pas partie de ceux disponibles." in str(form)

        options = form.find_all('option')
        assert len(options) == 3

        assert [o.attrs['value'] for o in options] == [
            '',
            str(association.role_set.get(name="Bénévole").id),
            str(association.role_set.get(name="Animateur").id),
        ]

        # 4 on tente de corriger un dirigeant mais on n'est pas en capacité

        engagement = association.engagement_set.create(
            user=foreign, role=association.role_set.get_dirigeant()
        )
        url = reverse(
            'board:association:engagement:update',
            args=[association.id, engagement.id],
        )
        response = client.get(url, follow=True)
        options = (
            bs(response.content, 'html.parser').find('form').find_all('option')
        )

        assert len(options) == 2
        assert [o.attrs['value'] for o in options] == [
            '',
            str(association.role_set.get_dirigeant().id),
        ]

    def test_self_erase_go_back_to_home(
        self, association, client, user, manager
    ):
        client.force_login(user)

        assert association.can_manage_engagements(user)

        data = {'confirm': True}
        response = client.post(
            reverse(
                'board:association:engagement:delete',
                args=[association.id, manager.id],
            ),
            data,
            follow=True,
        )

        assert not association.can_manage_engagements(user)

        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

        assert count_text_in_content(response, "Mes engagements associatifs")


@pytest.mark.django_db
class TestUserList:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 200), ('staff', 404), ('foreign', 404)],
    )
    def test_get_users(
        self,
        association,
        client,
        user,
        manager,
        superuser,
        foreign,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('board:association:user', args=[association.id])
        )
        assert response.status_code == expected

        if response.status_code == 200:

            table_row = (
                bs(response.content, 'html.parser')
                .find('tbody')
                .find_all('tr')
            )
            assert len(table_row) == 1
            assert "Dirigeant" in str(table_row[-1])
            assert "user@example.org" in str(table_row[-1])


@pytest.mark.django_db
class TestBenevalo:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 200), ('staff', 404), ('foreign', 404)],
    )
    def test_get_index(
        self,
        association,
        client,
        user,
        manager,
        superuser,
        foreign,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('board:association:benevalo:index', args=[association.id])
        )
        assert response.status_code == expected

    def test_index_table_footer(
        self, client, association, category, user, engagement
    ):
        user.benevalo_set.create(
            title="test 1",
            duration=timedelta(0, 1260),
            distance=21,
            execution_date=date.today(),
            association=association,
            category=category,
        )

        user.benevalo_set.create(
            title="test 2",
            duration=timedelta(0, 1260),
            distance=21,
            execution_date=date.today() - timedelta(1),
            association=association,
            category=category,
        )

        client.force_login(user)
        response = client.get(
            reverse(
                'board:association:self-benevalo:index', args=[association.id]
            )
        )
        assert response.status_code == 200
        table_foot = bs(response.content, 'html.parser').find('tfoot')
        assert "42 minutes" in str(table_foot)
        assert "42,00 km" in str(table_foot)
        assert "2 éléments" in str(table_foot)
        assert "sur 1 jour" in str(table_foot)

    def test_index_table_export(
        self, client, association, project, level, category, user, engagement
    ):
        user.benevalo_set.create(
            title="test 1",
            duration=timedelta(0, 1260),
            distance=21,
            execution_date=date.today(),
            association=association,
            category=category,
        )

        user.benevalo_set.create(
            title="test 2",
            duration=timedelta(0, 1260),
            distance=21,
            execution_date=date.today() - timedelta(1),
            execution_end_date=date.today() + timedelta(1),
            association=association,
            category=category,
            level=level,
            project=project,
            description='mise sous plis des convocations',
            is_active=False,
        )

        client.force_login(user)
        response = client.get(
            reverse(
                'board:association:self-benevalo:index', args=[association.id]
            )
            + '?_export=json'
        )
        assert response.status_code == 200
        assert response.json() == [
            {
                'Titre': "test 1",
                'Durée': "0:21:00",
                'Distance': "21,00",
                'Date': date.today().isoformat(),
                'Association': str(association),
                'Bénévole': str(user),
                'Catégorie': str(category),
                'Niveau': None,
                'Description': None,
                'Projet': None,
                'Date de fin': None,
                'En attente de modération': False,
            },
            {
                'Titre': "test 2",
                'Durée': "0:21:00",
                'Distance': "21,00",
                'Date': (date.today() - timedelta(1)).isoformat(),
                'Association': str(association),
                'Bénévole': str(user),
                'Catégorie': str(category),
                'Niveau': str(level),
                'Description': 'mise sous plis des convocations',
                'Projet': str(project),
                'Date de fin': (date.today() + timedelta(1)).isoformat(),
                'En attente de modération': True,
            },
        ]

    def test_post_create_update_delete(
        self, client, association, category, manager, user, foreign
    ):
        client.force_login(user)

        # 1. on fetch les benevalos de l'asso
        url = reverse(
            'board:association:benevalo:index', args=[association.id]
        )
        response = client.get(url)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Aucune donnée disponible." in str(table_row[-1])

        # 2. on créé un benevalo dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {
            'user': user.id,
            'title': 'benevalo de test',
            'duration': '42',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "benevalo de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« benevalo de test » a été ajouté" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "benevalo de test" in str(table_row[-1])

        # 3. on tente l'update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {
            'user': foreign.id,
            'title': 'benevalo de lutte',
            'duration': '42',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        assert count_text_in_content(
            response, "Ce choix ne fait pas partie de ceux disponibles."
        )
        execution_date = (
            bs(response.content, 'html.parser')
            .find(id='id_execution_date')
            .attrs['value']
        )
        assert execution_date == date.today().strftime('%Y-%m-%d')

        # 4. Bien reçu, on retente en intégrant le foreign user
        association.engagement_set.create(
            user=foreign, role=association.role_set.get_benevole()
        )
        response = client.post(url, data, follow=True)

        assert not count_text_in_content(response, "benevalo de test")
        assert count_text_in_content(response, "benevalo de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "benevalo de lutte" in str(table_row[-1])

        # 5. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Aucune donnée disponible." in str(table_row[-1])


@pytest.mark.django_db
class TestEngagedBenevalo:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 200), ('staff', 404), ('foreign', 404)],
    )
    def test_get_index(
        self,
        association,
        client,
        user,
        manager,
        superuser,
        foreign,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse(
                'board:association:engagement:benevalo',
                args=[association.id, association.get_engagement(user).id],
            )
        )
        assert response.status_code == expected
