from django.conf import settings
from django.contrib import messages
from django.contrib.messages.views import SuccessMessageMixin
from django.core.exceptions import ValidationError
from django.http import Http404
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse, reverse_lazy
from django.utils.text import slugify
from django.views import generic

from cruditor import views
from django_tables2.export.export import TableExport
from django_tables2.export.views import ExportMixin

from benevalibre.association import forms as asso_forms
from benevalibre.association import models as asso_models
from benevalibre.association.views import (
    AssociatedEditionMixin,
    AssociatedViewMixin,
)
from benevalibre.benevalo.models import Benevalo
from benevalibre.utils.html import fa_icon
from benevalibre.utils.views import (
    BaseCruditorMixin,
    PageMixin,
    PublicCruditorMixin,
)

from . import forms, tables


class ExportTable(ExportMixin):
    def get_export_filename(self, export_format):
        return "{}.{}".format(slugify(self.get_title()), export_format)

    def create_export(self, export_format):
        """
        We overload create_export because of a needed ajustement
        """
        exporter = TableExport(
            export_format=export_format,
            table=self.table_export_class(
                self.get_queryset(), **self.get_table_kwargs()
            ),
            exclude_columns=self.exclude_columns,
        )
        return exporter.response(
            filename=self.get_export_filename(export_format)
        )

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['export'] = [
            ['table', 'Format tableur', 'ods'],
            ['align-left', 'Format csv', 'csv'],
            ['code', 'Format json', 'json'],
        ]
        return context


class AssociationManageMenu:
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if hasattr(self, 'association'):
            association = self.association
        elif hasattr(self, 'object') and isinstance(
            self.object, asso_models.Association
        ):
            association = self.object
        else:
            raise NotImplementedError("No way to get any related association")

        capabilities = []
        if association.can_manage_engagements(self.request.user):
            capabilities.append(
                [
                    'users',
                    "Gérer les bénévoles",
                    'board:association:engagement:index',
                ]
            )
        elif association.can_list_users(self.request.user):
            capabilities.append(
                ['users', "Voir les bénévoles", 'board:association:user']
            )
        if association.can_manage_benevalos(self.request.user):
            capabilities.append(
                [
                    'gift',
                    "Gérer les bénévolats",
                    'board:association:benevalo:index',
                ]
            )
        if association.can_manage_levels(self.request.user):
            capabilities.append(
                [
                    'tachometer',
                    "Gérer les niveaux de bénévolat",
                    'board:association:level:index',
                ]
            )
        if association.can_manage_projects(self.request.user):
            capabilities.append(
                [
                    'wrench',
                    "Gérer les projets de l'association",
                    'board:association:project:index',
                ]
            )
        if association.can_manage_categories(self.request.user):
            capabilities.append(
                [
                    'tags',
                    "Gérer les catégories de bénévolat",
                    'board:association:category:index',
                ]
            )
        if association.can_manage_roles(self.request.user):
            capabilities.append(
                [
                    'key-modern',
                    "Gérer les roles de l'association",
                    'board:association:role:index',
                ]
            )
        if association.can_manage_association(self.request.user):
            capabilities.append(
                ['edit', "Modifier l'association", 'board:association:update']
            )
        context['capabilities'] = capabilities
        context['association'] = association
        return context


# ----------------------------------------------------------------------------
# Board
# ----------------------------------------------------------------------------


class Home(generic.RedirectView):
    def get_redirect_url(self, *args, **kwargs):
        """Dispatch between Board and AssociationIndex depending on things
        user has to see in Board"""
        if (
            not self.request.user.is_anonymous
            and self.request.user.engagement_set.exists()
        ):
            return reverse('board:board')
        else:
            return reverse('board:association:index')


class AssociationEngage(
    AssociatedViewMixin,
    AssociationManageMenu,
    BaseCruditorMixin,
    SuccessMessageMixin,
    generic.FormView,
):
    http_method_names = ['get', 'post']
    success_url = reverse_lazy('board:home')
    unsuccess_url = reverse_lazy('board:association:index')
    form_class = forms.AssociationEngageForm

    def get(self, request, *args, **kwargs):
        return redirect(self.unsuccess_url)

    def form_valid(self, form):
        try:
            """We try the setup the requestor with the local benevole role"""
            role = self.association.role_set.get_benevole()
            self.association.engagement_set.create(
                user=self.request.user,
                role=role,
                is_active=not self.association.moderate_engagement,
            )
            if self.association.moderate_engagement:
                # FIXME: notifier les modérateurs
                self.success_message = (
                    "Votre demande a bien été reçue et soumise à modération. "
                    "Les modérateurs de l'association donneront suite "
                    "dès que possible."
                )
            else:
                self.success_message = "Votre demande a été acceptée."
        except ValidationError as e:
            form.add_error(None, e)
            for k, v in form.errors.items():
                messages.error(self.request, v)
            return redirect(self.unsuccess_url)
        return super().form_valid(form)


class EngagementModerationMixin:
    def get_engagement_moderation_queryset(self):
        return (
            asso_models.Engagement.objects.manageable_by(self.request.user)
            .inactive()
            .distinct()
        )

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        qs = self.get_engagement_moderation_queryset()
        if qs.exists():
            context[
                'engagement_moderation_table'
            ] = tables.EngagementModerationTable(qs)
        return context


class BenevaloModerationMixin:
    def get_benevalo_moderation_queryset(self):
        return (
            Benevalo.objects.manageable_by(self.request.user)
            .inactive()
            .distinct()
        )

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        qs = self.get_benevalo_moderation_queryset()
        if qs.exists():
            context[
                'benevalo_moderation_table'
            ] = tables.BenevaloModerationTable(qs)
        return context


class AssociationModerationMixin:
    def get_association_moderation_queryset(self):
        return asso_models.Association.objects.inactive().distinct()

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if self.request.user.is_superuser:
            qs = self.get_association_moderation_queryset()
            if qs.exists():
                context[
                    'association_moderation_table'
                ] = tables.AssociationModerationTable(qs)
        return context


class EngagementIndex(
    BaseCruditorMixin,
    AssociationModerationMixin,
    BenevaloModerationMixin,
    EngagementModerationMixin,
    views.CruditorListView,
):
    title = "Mes engagements associatifs"
    table_class = tables.MyEngagementTable
    model = asso_models.Engagement

    def get_titlebutton_condition(self):
        return settings.OPEN_REGISTRATION

    def get_titlebutton_url(self):
        return reverse('board:association:create')

    def get_titlebutton_label(self):
        return fa_icon('plus-circle', "Enregistrer une association")

    def get_queryset(self):
        return self.model.objects.for_user(self.request.user)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['hide_doc'] = True
        return context


# ----------------------------------------------------------------------------
# Association management
# ----------------------------------------------------------------------------


class AssociationIndex(PublicCruditorMixin, views.CruditorListView):
    title = "Bienvenue à bord"
    table_class = tables.AssociationIndexTable
    model = asso_models.Association

    # FIXME: peuvent être en modération
    def get_titlebutton_condition(self):
        return settings.OPEN_REGISTRATION

    def get_titlebutton_url(self):
        return reverse('board:association:create')

    def get_titlebutton_label(self):
        return fa_icon('plus-circle', "Enregistrer une association")


class AssociationCreate(BaseCruditorMixin, views.CruditorAddView):
    model = asso_models.Association
    form_class = asso_forms.CreateForm

    def dispatch(self, request, *args, **kwargs):
        if not settings.OPEN_REGISTRATION:
            raise Http404
        return super().dispatch(request, *args, **kwargs)

    def check_multiple(self, form):
        warned = self.request.session.get('warned', [])
        if (
            self.object.name not in warned
            and self.model.objects.filter(name=self.object.name).exists()
        ):
            messages.warning(
                self.request,
                "Une association existe déjà sous ce nom. Êtes vous certain "
                "de vouloir poursuivre la création ?",
            )
            warned.append(self.object.name)
            self.request.session['warned'] = warned
            """
            Re-render the page squeezing CruditorMixin's form_invalid.
            """
            return self.render_to_response(
                self.get_context_data(form=form, formset_errors=False)
            )
        elif self.object.name in warned:
            """
            User has been warned, he force.
            """
            warned.remove(self.object.name)
            self.request.session['warned'] = warned

    def form_valid(self, form):
        self.object = form.save(commit=False)
        resend_form = self.check_multiple(form)
        if resend_form:
            return resend_form
        self.object = form.save()
        if settings.MODERATED_REGISTRATION:
            self.success_message = (
                "Votre demande a bien été reçue et soumise à modération. "
                "Les modérateurs de l'instance donneront suite "
                "dès que possible."
            )
            # FIXME: notifier les admins
        """We setup the creator with the local administrator role"""
        role = self.object.role_set.get_dirigeant()
        self.object.engagement_set.create(user=self.request.user, role=role)
        return super().form_valid(form)

    def get_success_url(self):
        if settings.MODERATED_REGISTRATION:
            return reverse('board:home')
        else:
            return reverse('board:association:detail', args=[self.object.pk])


class AssociationDetail(
    PageMixin, AssociationManageMenu, BaseCruditorMixin, generic.DetailView
):
    template_name = 'board/association_detail.html'
    page_title = "Détails d'une association"
    model = asso_models.Association


class AssociationUpdate(
    AssociationManageMenu, BaseCruditorMixin, views.CruditorChangeView
):
    form_class = asso_forms.UpdateForm
    page_title = "Modifier une association"
    model = asso_models.Association

    def get_object(self, **kwargs):
        asso = super().get_object(**kwargs)
        if not asso.can_manage_association(self.request.user):
            raise Http404
        return asso

    def get_success_url(self):
        return reverse('board:association:detail', args=[self.object.pk])

    def get_delete_url(self):
        return reverse('board:association:delete', args=[self.object.pk])

    def form_valid(self, form):
        self.object = form.save()
        if not self.object.is_active and self.request.user.is_superuser:
            self.object.is_active = True
            # FIXME: notifier le créateur
        return super().form_valid(form)


class AssociationDelete(
    AssociationManageMenu, BaseCruditorMixin, views.CruditorDeleteView
):
    page_title = "Supprimer une association"
    model = asso_models.Association

    def get_object(self, **kwargs):
        asso = super().get_object(**kwargs)
        if not asso.can_manage_association(self.request.user):
            raise Http404
        return asso

    def get_success_url(self):
        return reverse('board:home')


# ----------------------------------------------------------------------------
#  Category management
# ----------------------------------------------------------------------------


class AssociatedCategoryMixin(
    AssociatedViewMixin, AssociationManageMenu, BaseCruditorMixin
):
    model = asso_models.Category

    def dispatch(self, request, *args, **kwargs):
        if not self.association.can_manage_categories(self.request.user):
            raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return self.association.category_set.all()

    def get_success_url(self):
        return reverse(
            'board:association:category:index', args=[self.association.id]
        )


class AssociatedCategoryIndex(AssociatedCategoryMixin, views.CruditorListView):
    title = "Liste des categories de bénévolat"
    table_class = tables.CategoryIndexTable

    def get_titlebutton_condition(self):
        return True

    def get_titlebutton_url(self):
        return reverse(
            'board:association:category:create', args=[self.association.id]
        )

    def get_titlebutton_label(self):
        return fa_icon('plus-circle', "Ajouter une categorie de bénévolat")


class AssociatedCategoryCreate(
    AssociatedCategoryMixin, AssociatedEditionMixin, views.CruditorAddView
):
    form_class = asso_forms.CategoryCreateForm


class AssociatedCategoryUpdate(
    AssociatedCategoryMixin, AssociatedEditionMixin, views.CruditorChangeView
):
    form_class = asso_forms.CategoryUpdateForm

    def get_delete_url(self):
        return reverse(
            'board:association:category:delete',
            args=(self.association.id, self.object.pk),
        )


class AssociatedCategoryDelete(
    AssociatedCategoryMixin, views.CruditorDeleteView
):
    pass


# ----------------------------------------------------------------------------
#  Project management
# ----------------------------------------------------------------------------


class AssociatedProjectMixin(
    AssociatedViewMixin, AssociationManageMenu, BaseCruditorMixin
):
    model = asso_models.Project

    def dispatch(self, request, *args, **kwargs):
        if not self.association.can_manage_projects(self.request.user):
            raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return self.association.project_set.all()

    def get_success_url(self):
        return reverse(
            'board:association:project:index', args=[self.association.id]
        )


class AssociatedProjectIndex(AssociatedProjectMixin, views.CruditorListView):
    title = "Liste des projets"
    table_class = tables.ProjectIndexTable

    def get_titlebutton_condition(self):
        return True

    def get_titlebutton_url(self):
        return reverse(
            'board:association:project:create', args=[self.association.id]
        )

    def get_titlebutton_label(self):
        return fa_icon('plus-circle', "Ajouter un projet à l'association")


class AssociatedProjectCreate(
    AssociatedProjectMixin, AssociatedEditionMixin, views.CruditorAddView
):
    form_class = asso_forms.ProjectCreateForm


class AssociatedProjectUpdate(
    AssociatedProjectMixin, AssociatedEditionMixin, views.CruditorChangeView
):
    form_class = asso_forms.ProjectUpdateForm

    def get_delete_url(self):
        return reverse(
            'board:association:project:delete',
            args=(self.association.id, self.object.pk),
        )


class AssociatedProjectDelete(
    AssociatedProjectMixin, views.CruditorDeleteView
):
    pass


# ----------------------------------------------------------------------------
#  Role management
# ----------------------------------------------------------------------------


class AssociatedRoleMixin(
    AssociatedViewMixin, AssociationManageMenu, BaseCruditorMixin
):
    model = asso_models.Role

    def dispatch(self, request, *args, **kwargs):
        if not self.association.can_manage_roles(self.request.user):
            raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return self.association.role_set.all()

    def get_success_url(self):
        # We may have lost our capability
        if not self.association.can_manage_roles(self.request.user):
            return reverse('board:board')
        return reverse(
            'board:association:role:index', args=[self.association.id]
        )


class AssociatedRoleIndex(AssociatedRoleMixin, views.CruditorListView):
    title = "Liste des rôles des bénévoles"
    table_class = tables.RoleIndexTable

    def get_titlebutton_condition(self):
        return True

    def get_titlebutton_url(self):
        return reverse(
            'board:association:role:create', args=[self.association.id]
        )

    def get_titlebutton_label(self):
        return fa_icon('plus-circle', "Ajouter un rôle à l'association")


class AssociatedRoleCreate(
    AssociatedRoleMixin, AssociatedEditionMixin, views.CruditorAddView
):
    form_class = asso_forms.RoleCreateForm


class AssociatedRoleUpdate(
    AssociatedRoleMixin, AssociatedEditionMixin, views.CruditorChangeView
):
    form_class = asso_forms.RoleUpdateForm

    def get_delete_url(self):
        return reverse(
            'board:association:role:delete',
            args=(self.association.id, self.object.pk),
        )


class AssociatedRoleDelete(AssociatedRoleMixin, views.CruditorDeleteView):
    pass


# ----------------------------------------------------------------------------
#  Level management
# ----------------------------------------------------------------------------


class AssociatedLevelMixin(
    AssociatedViewMixin, AssociationManageMenu, BaseCruditorMixin
):
    model = asso_models.Level

    def dispatch(self, request, *args, **kwargs):
        if not self.association.can_manage_levels(self.request.user):
            raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return self.association.level_set.all()

    def get_success_url(self):
        return reverse(
            'board:association:level:index', args=[self.association.id]
        )


class AssociatedLevelIndex(AssociatedLevelMixin, views.CruditorListView):
    title = "Liste des niveaux de bénévolat"
    table_class = tables.LevelIndexTable

    def get_titlebutton_condition(self):
        return True

    def get_titlebutton_url(self):
        return reverse(
            'board:association:level:create', args=[self.association.id]
        )

    def get_titlebutton_label(self):
        return fa_icon('plus-circle', "Ajouter un niveau de bénévolat")


class AssociatedLevelCreate(
    AssociatedLevelMixin, AssociatedEditionMixin, views.CruditorAddView
):
    form_class = asso_forms.LevelCreateForm


class AssociatedLevelUpdate(
    AssociatedLevelMixin, AssociatedEditionMixin, views.CruditorChangeView
):
    form_class = asso_forms.LevelUpdateForm

    def get_delete_url(self):
        return reverse(
            'board:association:level:delete',
            args=(self.association.id, self.object.pk),
        )


class AssociatedLevelDelete(AssociatedLevelMixin, views.CruditorDeleteView):
    pass


# ----------------------------------------------------------------------------
#  Engagement listing
# ----------------------------------------------------------------------------


class AssociatedEngagementReadOnlyIndex(
    AssociatedViewMixin,
    AssociationManageMenu,
    BaseCruditorMixin,
    views.CruditorListView,
):
    model = asso_models.Engagement
    table_class = tables.EngagementReadOnlyIndexTable

    def dispatch(self, request, *args, **kwargs):
        if not self.association.can_list_users(self.request.user):
            raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return self.association.engagement_set.active()


# ----------------------------------------------------------------------------
#  Engagement management
# ----------------------------------------------------------------------------


class AssociatedEngagementMixin(
    AssociatedViewMixin, AssociationManageMenu, BaseCruditorMixin
):
    model = asso_models.Engagement

    def dispatch(self, request, *args, **kwargs):
        if not self.association.can_manage_engagements(self.request.user):
            raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return self.association.engagement_set.all()

    def get_success_url(self):
        # We may have lost our capability
        if not self.association.can_manage_engagements(self.request.user):
            return reverse('board:board')
        return reverse(
            'board:association:engagement:index', args=[self.association.id]
        )


class AssociatedEngagementIndex(
    AssociatedEngagementMixin, views.CruditorListView
):
    title = "Liste des bénévoles"
    table_class = tables.EngagementIndexTable


class AssociatedEngagementUpdate(
    AssociatedEngagementMixin, AssociatedEditionMixin, views.CruditorChangeView
):
    form_class = forms.AssociatedEngagementUpdateForm

    def get_delete_url(self):
        return reverse(
            'board:association:engagement:delete',
            args=(self.association.id, self.object.pk),
        )

    def form_valid(self, form):
        self.object = form.save()
        if not self.object.is_active:
            self.object.is_active = True
            # FIXME: notifier le bénévole
        return super().form_valid(form)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        user_role = self.association.get_role(self.request.user)
        if self.object.is_active:
            managed_user_role = self.association.get_role(self.object.user)
        else:
            managed_user_role = self.association.role_set.get_benevole()
        deleguables = self.association.role_set.delegables_by(user_role)

        if managed_user_role in deleguables:
            kwargs['role_set'] = deleguables
        else:
            kwargs['role_set'] = self.association.role_set.filter(
                pk=managed_user_role.pk
            )
        return kwargs


class AssociatedEngagementDelete(
    AssociatedEngagementMixin, views.CruditorDeleteView
):
    pass


# ----------------------------------------------------------------------------
# Benevalo management
# ----------------------------------------------------------------------------


class AssociatedFormMixin:
    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['association'] = self.association
        return kwargs


class AssociatedBenevaloMixin(AssociatedViewMixin, AssociationManageMenu):
    model = Benevalo

    def dispatch(self, request, *args, **kwargs):
        if not self.association.can_manage_benevalos(self.request.user):
            raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return self.association.benevalo_set.all()

    def get_success_url(self):
        return reverse(
            'board:association:benevalo:index', args=[self.association.id]
        )


class AssociatedBenevaloModerationMixin(BenevaloModerationMixin):
    def get_benevalo_moderation_queryset(self):
        return (
            self.association.benevalo_set.manageable_by(self.request.user)
            .inactive()
            .distinct()
        )


class AssociatedBenevaloIndex(
    AssociatedBenevaloMixin,
    ExportTable,
    AssociatedBenevaloModerationMixin,
    BaseCruditorMixin,
    views.CruditorListView,
):
    title = "Tout le bénévolat"
    table_class = tables.AssociatedBenevaloTable
    table_export_class = tables.BenevaloExportTable
    model = Benevalo

    def get_titlebutton_condition(self):
        return True

    def get_titlebutton_url(self):
        return reverse(
            'board:association:benevalo:create', args=[self.association.id]
        )

    def get_titlebutton_label(self):
        return fa_icon(
            'plus-circle',
            "Saisir un bénévolat pour {}".format(self.association),
        )


class AssociatedBenevaloCreate(
    AssociatedBenevaloMixin,
    AssociatedEditionMixin,
    AssociatedFormMixin,
    BaseCruditorMixin,
    views.CruditorAddView,
):
    form_class = forms.AssociatedBenevaloCreateForm


class AssociatedBenevaloUpdate(
    AssociatedBenevaloMixin,
    AssociatedFormMixin,
    BaseCruditorMixin,
    views.CruditorChangeView,
):
    form_class = forms.AssociatedBenevaloUpdateForm

    def form_valid(self, form):
        self.object = form.save()
        if not self.object.is_active:
            self.object.is_active = True
            # FIXME: notifier le bénévole
        return super().form_valid(form)

    def get_delete_url(self):
        return reverse(
            'board:association:benevalo:delete',
            args=(self.association.id, self.object.pk),
        )


class AssociatedBenevaloDelete(
    AssociatedBenevaloMixin, BaseCruditorMixin, views.CruditorDeleteView
):
    pass


# ----------------------------------------------------------------------------
# EngagedBenevalo management
# ----------------------------------------------------------------------------


class AssociatedEngagedBenevaloModerationMixin(BenevaloModerationMixin):
    def get_benevalo_moderation_queryset(self):
        return (
            self.association.benevalo_set.by_user(self.engagement.user)
            .manageable_by(self.request.user)
            .inactive()
            .distinct()
        )


class AssociatedEngagedBenevaloIndex(
    AssociatedViewMixin,
    AssociationManageMenu,
    AssociatedEngagedBenevaloModerationMixin,
    ExportTable,
    BaseCruditorMixin,
    views.CruditorListView,
):
    title = "Bénévolat"
    table_class = tables.AssociatedEngagedBenevaloTable
    table_export_class = tables.BenevaloExportTable
    model = Benevalo

    def setup(self, request, *args, **kwargs):
        super().setup(request, *args, **kwargs)
        self.engagement = self.get_engagement()

    def get_engagement(self):
        return get_object_or_404(
            asso_models.Engagement.objects.viewable_by(
                self.request.user
            ).distinct(),
            pk=self.kwargs['engagement'],
        )

    def dispatch(self, request, *args, **kwargs):
        if not self.association.can_manage_benevalos(
            self.request.user
        ) or not self.association.can_self_list_benevalo(self.engagement.user):
            raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_title(self):
        return "{} de {}".format(super().get_title(), self.engagement.user)

    def get_titlebutton_condition(self):
        return self.association.is_engaged(self.engagement.user)

    def get_titlebutton_url(self):
        return reverse(
            'board:association:benevalo:create', args=[self.association.id]
        )

    def get_titlebutton_label(self):
        return fa_icon(
            'plus-circle',
            "Saisir un bénévolat de {} pour {}".format(
                self.engagement.user, self.association
            ),
        )

    def get_queryset(self):
        return self.association.benevalo_set.by_user(self.engagement.user)


# ----------------------------------------------------------------------------
# Benevalo self management
# ----------------------------------------------------------------------------


class AssociatedSelfBenevaloMixin(AssociatedViewMixin):
    model = Benevalo

    def dispatch(self, request, *args, **kwargs):
        if not self.association.can_self_create_benevalo(self.request.user):
            raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return self.association.benevalo_set.by_user(self.request.user)

    def get_success_url(self):
        return reverse(
            'board:association:self-benevalo:index', args=[self.association.id]
        )


class AssociatedSelfBenevaloIndex(
    AssociatedViewMixin,
    AssociationManageMenu,
    AssociatedBenevaloModerationMixin,
    ExportTable,
    BaseCruditorMixin,
    views.CruditorListView,
):
    title = "Mon bénévolat"
    table_class = tables.AssociatedEngagedSelfBenevaloTable
    table_export_class = tables.BenevaloExportTable
    model = Benevalo

    def dispatch(self, request, *args, **kwargs):
        if not self.association.can_self_list_benevalo(self.request.user):
            raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_titlebutton_condition(self):
        return self.association.can_self_create_benevalo(self.request.user)

    def get_titlebutton_url(self):
        return reverse(
            'board:association:self-benevalo:create',
            args=[self.association.id],
        )

    def get_titlebutton_label(self):
        return fa_icon(
            'plus-circle',
            "Saisir un bénévolat pour {}".format(self.association),
        )

    def get_queryset(self):
        return self.association.benevalo_set.by_user(self.request.user)


class AssociatedSelfBenevaloCreate(
    AssociatedSelfBenevaloMixin,
    AssociatedFormMixin,
    BaseCruditorMixin,
    views.CruditorAddView,
):
    form_class = forms.AssociatedEngagedBenevaloCreateForm

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.association = self.association
        self.object.user = self.request.user

        if self.object.association.need_moderation(self.request.user):
            self.object.is_active = False
            self.success_message = (
                "Votre saisie a bien été reçue et soumise à modération. "
                "Les animateurs de l'association donneront suite "
                "dès que possible."
            )
            # FIXME: notifier les anims

        self.object.save()

        return super().form_valid(form)


class AssociatedSelfBenevaloUpdate(
    AssociatedSelfBenevaloMixin,
    AssociatedFormMixin,
    BaseCruditorMixin,
    views.CruditorChangeView,
):
    form_class = forms.AssociatedEngagedBenevaloUpdateForm

    def get_delete_url(self):
        return reverse(
            'board:association:self-benevalo:delete',
            args=(self.association.id, self.object.pk),
        )


class AssociatedSelfBenevaloDelete(
    AssociatedSelfBenevaloMixin, BaseCruditorMixin, views.CruditorDeleteView
):
    pass
