from django.core.exceptions import ValidationError
from django.db import models

import reversion

from benevalibre.utils.mixins import HTMLDocString
from benevalibre.utils.models import AbstractRole, AbstractTaxonomy


@reversion.register()
class InstanceCategory(HTMLDocString, AbstractTaxonomy):
    """
    <p>
        Les catégories servent à qualifier les actions de bénévolat. Mais
        elles sont propres à chaque association, qui peut personnaliser sa
        liste de catégories.
        <br>
        Afin de rendre possible des statistiques inter-associatives, elles sont
        donc chacune attachées à une «&nbsp;catégorie d'instance&nbsp;».
        <br>
        Par conséquent il n'est pas recommandé de modifier ces groupes sans
        concertation préalable avec les fédérations de mouvements associatifs.
    </p>
    """

    class Meta:
        verbose_name = "catégorie d'instance"
        verbose_name_plural = "catégories d'instance"


class AbstractLevel(AbstractTaxonomy):
    class Meta:
        abstract = True


@reversion.register()
class DefaultLevel(HTMLDocString, AbstractLevel):
    """
    <p>
        Les «&nbsp;niveaux par défaut&nbsp;» sont crées et affectés
        par défaut à toute nouvelle association inscrite sur la plateforme.
        <br>
        Les niveaux de bénévolat permettent pour ceux qui souhaitent de
        caractériser le niveau d'expertise que l'action mobilise. C'est utile
        par exemple pour les associations qui souhaitent moduler la
        valorisation en fonction de niveaux d'expertise différents.
    </p>
    """

    class Meta:
        verbose_name = "niveau par défaut"
        verbose_name_plural = "niveaux par défaut"


class AbstractCategory(AbstractTaxonomy):
    class Meta:
        abstract = True

    instance_category = models.ForeignKey(
        InstanceCategory,
        verbose_name="catégorie d'instance",
        on_delete=models.PROTECT,
    )


@reversion.register()
class DefaultCategory(HTMLDocString, AbstractCategory):
    """
    <p>
        Les «&nbsp;catégories par défaut&nbsp;» sont crées et affectés
        par défaut à toute nouvelle association inscrite sur la plateforme.
        <br>
        Afin de rendre possible des statistiques inter-associatives, elles
        sont chacune attachées à une «&nbsp;catégorie d'instance&nbsp;».
        Ainsi, chaque association peut personnaliser sa liste de catégories.
    </p>
    """

    class Meta:
        verbose_name = "catégorie par défaut"
        verbose_name_plural = "catégories par défaut"


class DefaultRole(HTMLDocString, AbstractRole):
    """
    <p>
        Les «&nbsp;rôles par défaut&nbsp;» sont crées et affectés
        par défaut à toute nouvelle association inscrite sur la plateforme.
    </p>
    <p>
        Les rôles définissent les permissions dont jouissent chaque bénévole.
        Ces permissions sont, par ordre d'importance croissante&nbsp;:
        <ul>
            <li>
                Lister les utilisateurs&nbsp;: donne au bénévole la possibilité
                de voir qui snt les autes bénévoles engagés dans l'association.
            </li>
            <li>
                Délègue ses permissions&nbsp;: donne au bénévole la possibilité
                de gérer les rôles des bénévoles et de transférer son rôle aux
                autres bénévoles. Ce rôle implique de lister les autres
                bénévoles engagés dans l'association.
            </li>
            <li>
                Gère les bénévolats&nbsp;: donne au bénévole la possibilité de
                modérer, saisir, corriger ou effacer le bénévolat des autres
                bénévoles. Ce rôle implique de lister les autres bénévoles
                engagés dans l'association.
            </li>
            <li>
                Gère les niveaux, projet, catégories&nbsp;: donne
                respectivement au bénévole la capacité de gérer respectivement
                les niveaux, les projets et les catégories.
            </li>
            <li>
                Gère les rôles&nbsp;: donne au bénévole la capacité de gérer
                les rôles. Ce rôle permet un accès total à tous les rôles de
                l'association.
            </li>
            <li>
                Gère l'association&nbsp;: donne au bénévole la capacité de
                gérer la totalité des attributs de l'association.
            </li>
        </ul>
        Le «&nbsp;rôle des nouveaux arrivants&nbsp;» est le rôle associé aux
        bénévole lorsque ceux-ci rejoingnent l'association.
    </p>
    """

    class Meta:
        verbose_name = "rôle par défaut"
        verbose_name_plural = "rôles par défaut"

    def save(self, *args, **kwargs):
        self.clean()
        super().save(*args, **kwargs)

    def clean(self):
        if (
            self.new_comers
            and DefaultRole.objects.filter(
                ~models.Q(id=self.id), new_comers=True
            ).exists()
        ):
            raise ValidationError(
                "Un autre rôle possède déjà l'attribut « nouvel arrivant ». "
                "Il ne peut n'y en avoir qu'un."
            )
        elif (
            not self.new_comers
            and not DefaultRole.objects.filter(
                ~models.Q(id=self.id), new_comers=True
            ).exists()
        ):
            raise ValidationError(
                "Aucun autre rôle ne possède l'attribut « nouvel arrivant ». "
                "Vous devez en disposer d'un pour permettre l'inscription d'un "
                "bénévole."
            )
