from django.contrib.messages import get_messages
from django.urls import reverse

import pytest
from bs4 import BeautifulSoup as bs

from benevalibre.utils.tests import count_text_in_content


@pytest.mark.django_db
class TestInstanceCategories:
    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, "<!--Please Login-->"),
            ('user', "<!--Please Login-->"),
            ('staff', "Liste des "),
        ],
    )
    def test_get_index(self, client, user, superuser, use, expected):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(reverse('instance:instance_category:index'))
        assert response.status_code == 200
        assert count_text_in_content(response, expected)

    def test_post_create_update_delete(self, client, user, superuser):
        # 0. on fetch les catégories d'instance
        client.force_login(superuser)
        response = client.get(reverse('instance:instance_category:index'))

        # 1. on créé une catégories d'instance
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'categorie de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "categorie de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« categorie de test » a été ajouté" in messages[0]

        # 2. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'categorie de lutte'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« categorie de lutte » a été modifié" in messages[0]

        # 3. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« categorie de lutte » a été supprimé" in messages[0]


@pytest.mark.django_db
class TestDefaultCategories:
    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, "<!--Please Login-->"),
            ('user', "<!--Please Login-->"),
            ('staff', "Liste des "),
        ],
    )
    def test_get_index(self, client, user, superuser, use, expected):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(reverse('instance:category:index'))
        assert response.status_code == 200
        assert count_text_in_content(response, expected)

    def test_post_create_update_delete(self, client, user, superuser):
        # 0. on fetch les catégories
        client.force_login(superuser)
        response = client.get(reverse('instance:category:index'))

        # 1. on créé une catégorie
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'categorie de test', 'instance_category': 1}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "categorie de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« categorie de test » a été ajouté" in messages[0]

        # 2. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'categorie de lutte', 'instance_category': 1}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« categorie de lutte » a été modifié" in messages[0]

        # 3. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« categorie de lutte » a été supprimé" in messages[0]


@pytest.mark.django_db
class TestDefaultRoles:
    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, "<!--Please Login-->"),
            ('user', "<!--Please Login-->"),
            ('staff', "Liste des "),
        ],
    )
    def test_get_index(self, client, user, superuser, use, expected):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(reverse('instance:role:index'))
        assert response.status_code == 200
        assert count_text_in_content(response, expected)

    def test_post_create_update_delete(self, client, user, superuser):
        # 0. on fetch les rôles
        client.force_login(superuser)
        response = client.get(reverse('instance:role:index'))

        # 1. on créé un rôle
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'rôle de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "rôle de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« rôle de test » a été ajouté" in messages[0]

        # 2. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'rôle de lutte'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« rôle de lutte » a été modifié" in messages[0]

        # 3. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« rôle de lutte » a été supprimé" in messages[0]


@pytest.mark.django_db
class TestDefaultLevels:
    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, "<!--Please Login-->"),
            ('user', "<!--Please Login-->"),
            ('staff', "Liste des "),
        ],
    )
    def test_get_index(self, client, user, superuser, use, expected):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(reverse('instance:level:index'))
        assert response.status_code == 200
        assert count_text_in_content(response, expected)

    def test_post_create_update_delete(self, client, user, superuser):
        # 0. on fetch les niveaux
        client.force_login(superuser)
        response = client.get(reverse('instance:level:index'))

        # 1. on créé un niveau
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'niveau de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "niveau de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« niveau de test » a été ajouté" in messages[0]

        # 2. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'niveau de lutte'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« niveau de lutte » a été modifié" in messages[0]

        # 3. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', class_='btn-danger')
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« niveau de lutte » a été supprimé" in messages[0]
