import os.path
from io import BytesIO

from django.core.files.images import ImageFile

from willow.image import Image as WillowImage


def resize_image_to_file(
    image,
    size,
    output_name=None,
    output_format=None,
    keep_aspect=True,
    **kwargs
):
    """
    Redimensionne une image vers un fichier.

    Crée et retourne une instance ``django.core.files.File`` à partir
    de `image`, généralement un champ de type ``ImageField``. L'image
    générée sera redimensionnée suivant les dimensions `size`, en
    respectant les proportions si ``keep_aspect`` vaut ``True``.

    ``output_name`` sera utilisé comme nom de fichier, ou sera récupéré
    depuis `image.name`.

    Si ``output_format`` n'est pas spécifié, le format de l'image ``image``
    sera préservé. Les arguments supplémentaires seront passés à la
    méthode d'enregistrement (voir la documentation de `Willow`).
    """
    if output_format is not None and output_format not in (
        'jpeg',
        'png',
        'gif',
        'webp',
    ):
        raise ValueError("Unknown output image format: %s" % output_format)

    if not output_name:
        output_name = os.path.splitext(os.path.basename(image.name))[0]

    # ouvre l'image et définis le format de sortie
    output = WillowImage.open(image)
    output_format = output_format or output.format_name

    # redimensionne au besoin l'image
    if keep_aspect:
        size = get_image_max_size(output.get_size(), size)
    if size != output.get_size():
        output = output.resize(size)

    # enregistre la nouvelle image dans un tampon
    operation_name = 'save_as_' + output_format
    output = getattr(output, operation_name)(BytesIO(), **kwargs)

    return ImageFile(output.f, name='{}.{}'.format(output_name, output_format))


def get_image_max_size(size, max_size):
    """
    Détermine la taille maximale et optimale d'une image.

    Tente de réduire une image avec comme dimensions ``size`` afin qu'elle
    ne soit pas plus grande que ``max_size``, en respectant les proportions.
    Si l'image peut être réduite, les nouvelles dimensions sont retournées.

    Les dimensions doivent être un tuple de 2 éléments : `(width, height)`.
    """
    for value in (size, max_size):
        if not isinstance(value, (tuple, list)) or not len(value) == 2:
            raise TypeError("'size' and 'max_size' must be a 2-tuple")

    width, height = size
    max_width, max_height = max_size

    if not max_width and not max_height:
        raise ValueError("Either the width or the height must be set")
    if max_width and width > max_width:
        height = int(max(height * max_width / width, 1))
        width = max_width
    if max_height and height > max_height:
        width = int(max(width * max_height / height, 1))
        height = max_height

    return (width, height)
