import os.path

from django.core.files.images import ImageFile

import pytest
from willow.image import Image as WillowImage

from ..images import get_image_max_size, resize_image_to_file


@pytest.mark.parametrize(
    'size, max_size, result',
    [
        ((100, 100), (200, 200), (100, 100)),
        ((200, 200), (100, 100), (100, 100)),
        ((200, 100), (100, 100), (100, 50)),
        ((100, 200), (100, 100), (50, 100)),
        ((200, 100), (100, None), (100, 50)),
        ((100, 200), (100, None), (100, 200)),
        ((100, 200), (None, 100), (50, 100)),
        ((200, 100), (None, 100), (200, 100)),
    ],
)
def test_get_image_max_size(size, max_size, result):
    assert get_image_max_size(size, max_size) == result


def test_get_image_max_size_type_error():
    with pytest.raises(TypeError):
        get_image_max_size(10, (10, 10))
    with pytest.raises(TypeError):
        get_image_max_size('10', (10, 10))
    with pytest.raises(TypeError):
        get_image_max_size((10, 10, 10), (10, 10))

    with pytest.raises(TypeError):
        get_image_max_size((10, 10), 10)


def test_get_image_max_size_max_error():
    with pytest.raises(ValueError):
        get_image_max_size((10, 10), (0, 0))
    with pytest.raises(ValueError):
        get_image_max_size((10, 10), (None, None))


# resize_image_to_file
# --------------------

FLOWER_PATH = os.path.join(os.path.dirname(__file__), 'images/flower.jpg')


@pytest.fixture
def flower():  # JPEG / 480x360
    image = ImageFile(open(FLOWER_PATH, 'rb'))
    image.size = (image.width, image.height)
    return image


class TestResizeImageToFile:
    def test_resized_output(self, flower):
        output = resize_image_to_file(flower, (48, 48))
        assert isinstance(output, ImageFile)
        assert output.name == 'flower.jpeg'
        assert output.size > 0
        assert output.width == 48
        assert output.height == 36

    def test_output_name(self, flower):
        output = resize_image_to_file(flower, flower.size, output_name='out')
        assert output.name == 'out.jpeg'

    def test_output_format(self, flower):
        output = resize_image_to_file(flower, flower.size, output_format='png')
        assert output.name == 'flower.png'
        output = WillowImage.open(output.open())
        assert output.format_name == 'png'

    def test_unknown_output_format(self, flower):
        with pytest.raises(ValueError):
            resize_image_to_file(flower, flower.size, output_format='tiff')

    def test_keep_aspect(self, flower):
        output = resize_image_to_file(flower, (48, 48), keep_aspect=False)
        assert output.width == 48
        assert output.height == 48
