from django.urls import reverse_lazy


class BaseCruditorMixin:
    """
    Mixin à appliquer aux vues cruditor. Il défini des paramètres
    globaux de l'application.
    Impose un utilisateur connecté, sinon renvoit vers l'écran de connexion.
    """

    menu_title = "Bénévalibre"
    index_url = reverse_lazy('board:home')
    login_url = reverse_lazy('accounts:login')
    logout_url = reverse_lazy('accounts:logout')
    change_password_url = reverse_lazy('accounts:password_reset')
    staff_required = False

    def get_queryset(self):
        if self.queryset is not None:
            qs = self.queryset
        elif hasattr(self, 'model'):
            qs = self.model.objects.all()
        else:
            raise NotImplementedError("No way to compute queryset")

        if hasattr(qs, 'viewable_by'):
            return qs.viewable_by(self.request.user).distinct()
        else:
            return qs

    def get_titlebutton_condition(self):
        return None

    def get_titlebutton_url(self):
        return None

    def get_titlebutton_label(self):
        return None

    def get_titlebuttons(self):
        if self.get_titlebutton_condition():
            return [
                {
                    'url': self.get_titlebutton_url(),
                    'label': self.get_titlebutton_label(),
                    'button_class': 'secondary',
                }
            ]


class StaffCruditorMixin(BaseCruditorMixin):
    """
    Vue cruditor réservée au staff
    """

    staff_required = True


class PublicCruditorMixin(BaseCruditorMixin):
    """
    Vue cruditor publique
    """

    def handle_not_logged_in(request, *args, **kwargs):
        return True


class PageMixin:
    """
    Mixin à appliquer aux vues présentant une page. Il ajoute au contexte un
    objet ``page`` facilitant son affichage et sa définition.
    """

    #: Le titre de la page
    page_title = None

    def get_page_title(self):
        """Retourne le titre de la page définis par ``page_title``."""
        return self.page_title

    def get_page_context(self):
        """Fournis le contexte à utiliser pour la page."""
        return {'title': self.get_page_title()}

    def get_context_data(self, **kwargs):
        """Ajoute la variable ``page`` au contexte du template."""
        context = super().get_context_data(**kwargs)
        context['page'] = self.get_page_context()
        return context
